import tempy from 'tempy';
import { homedir } from 'os';
import { basename, extname } from 'path';
import { nanoid } from 'nanoid';
export default class Download {
    file;
    imgType;
    flags;
    tempDirectory = tempy.directory();
    uniqueId = nanoid(10);
    constructor(file) {
        this.file = file;
    }
    expandHomeDirectory(saveDirectory) {
        if (saveDirectory.startsWith('~')) {
            return saveDirectory.replace('~', homedir());
        }
        return saveDirectory;
    }
    set setImgType(imgType) {
        this.imgType = imgType;
    }
    set setFlags(flags) {
        this.flags = flags;
    }
    get getSaveDirectory() {
        return this.flags.toClipboard
            ? this.tempDirectory
            : this.expandHomeDirectory(this.flags.saveTo);
    }
    get getOriginalFileName() {
        return this.file ? basename(this.file, extname(this.file)) : 'stdin';
    }
    get getNewFileName() {
        return this.flags.saveAs || `${this.getOriginalFileName}-${this.uniqueId}`;
    }
    get getDownloadedAsPath() {
        return `${this.getSaveDirectory}/carbon.${this.imgType}`;
    }
    get getSavedAsPath() {
        return `${this.getSaveDirectory}/${this.getNewFileName}.${this.imgType}`;
    }
    get getPath() {
        return this.flags.toClipboard
            ? this.getDownloadedAsPath
            : this.getSavedAsPath;
    }
}
