import fileExtension from 'file-extension';
import { EOL } from 'os';
import { rename } from 'node:fs/promises';
import extensionsMap from '../helpers/cli/extensions-map.helper.js';
export default class FileHandler {
    file;
    extensions = extensionsMap;
    constructor(file) {
        this.file = file;
    }
    async process(input, startLine = 0, endLine = 1000) {
        return new Promise((resolve, reject) => {
            if (startLine > endLine) {
                return reject(new Error('Nonsensical line numbers.'));
            }
            resolve(input
                .split(EOL)
                .filter((_, index) => {
                const currentLine = index + 1;
                return currentLine >= startLine && currentLine <= endLine;
            })
                .join(EOL));
        });
    }
    async rename(from, to) {
        await rename(from, to);
    }
    get getMimeType() {
        const extension = fileExtension(this.file, {
            preserveCase: true,
        });
        return this.extensions.get(extension) ?? 'auto';
    }
}
