import jsonFile from 'jsonfile';
import fileExists from 'file-exists';
import _ from 'lodash';
import { EOL } from 'os';
import { CONFIG_PATH, CONFIG_LATEST_PRESET, } from '../helpers/cli/constants.helper.js';
import presetMissingView from '../views/preset-missing.view.js';
export default class PresetHandler {
    configPath;
    ignoredSettings = [
        'save',
        'preset',
        'language',
        'highlight',
    ];
    constructor(configPath = CONFIG_PATH) {
        this.configPath = configPath;
    }
    async writeConfig(configSettings = {}, jsonFileOptions = {}) {
        await jsonFile.writeFileSync(this.configPath, configSettings, jsonFileOptions);
    }
    async readConfig() {
        const configPath = this.configPath;
        if (await fileExists(configPath)) {
            return await jsonFile.readFileSync(configPath);
        }
        // Only create a global config if none exists, hence don’t overwrite custom --config
        if (configPath === CONFIG_PATH) {
            await this.writeConfig();
            return await this.readConfig();
        }
        return {};
    }
    warn(preset) {
        console.warn(presetMissingView(preset));
    }
    async getPreset(preset) {
        const currentConfig = await this.readConfig();
        if (preset in currentConfig) {
            return currentConfig[preset];
        }
        this.warn(preset);
        return {};
    }
    async savePreset(preset = CONFIG_LATEST_PRESET, presetSettings = {}) {
        const whiteListedSettings = _.omit(presetSettings, this.ignoredSettings);
        const currentConfig = await this.readConfig();
        const upcomingConfig = {
            ...currentConfig,
            ...(preset !== CONFIG_LATEST_PRESET && {
                [preset]: whiteListedSettings,
            }),
            [CONFIG_LATEST_PRESET]: whiteListedSettings,
        };
        await this.writeConfig(upcomingConfig, {
            spaces: 2,
            EOL,
        });
    }
}
