import meow from 'meow';
import inquirer from 'inquirer';
import clipboard from 'clipboardy';
import _ from 'lodash';
import getStdin from 'get-stdin';
import { mappingsConfig, } from '../config/carbon/mappings.config.js';
import readFileAsync from '../utils/read-file-async.util.js';
import flags from '../config/cli/flags.config.js';
import promptConfig from '../config/cli/prompt.config.js';
import defaultView from '../views/default.view.js';
import usageErrorView from '../views/usage-error.view.js';
export default class Prompt {
    file;
    flags;
    input;
    answers;
    static async create() {
        const PromptInstance = new this();
        await PromptInstance.init();
        return PromptInstance;
    }
    async init() {
        [this.file, this.flags] = await this.initCLI();
        this.input = await this.readInput();
        this.answers = await this.initInteractiveMode();
    }
    async initCLI() {
        const cliHelper = meow(defaultView, {
            // TODO: Include this once Jest+ESM problem is fixed
            // importMeta: import.meta,
            flags,
        });
        return [cliHelper.input[0], cliHelper.flags];
    }
    async readInput() {
        let stdin;
        if (this.file) {
            return (await readFileAsync(this.file));
        }
        if (this.flags.fromClipboard) {
            return clipboard.readSync();
        }
        if ((stdin = await getStdin())) {
            return stdin;
        }
        console.error(usageErrorView);
        process.exit(1);
    }
    async initInteractiveMode() {
        if (this.flags.interactive) {
            return this.mapAnswersToCarbonValues(await inquirer.prompt(promptConfig));
        }
        return {};
    }
    mapAnswersToCarbonValues(unmappedAnswers) {
        return _.mapValues(unmappedAnswers, (value, key) => mappingsConfig[key]?.[value] ?? value);
    }
    get getFlags() {
        return this.flags;
    }
    get getAnswers() {
        return this.answers;
    }
    get getFile() {
        return this.file;
    }
    get getInput() {
        return this.input;
    }
}
