import { chromium, firefox, webkit, } from '@playwright/test';
import { CARBON_CUSTOM_THEME, CARBON_LOCAL_STORAGE_KEY, } from '../../src/helpers/carbon/constants.helper.js';
export default class Renderer {
    type;
    browser;
    page;
    pageOptions = {
        viewport: {
            width: 1800,
            height: 1000,
        },
        deviceScaleFactor: 2,
    };
    static async create(engineType = "chromium" /* CarbonCLIEngineFlagEnum.chromium */, disableHeadless = false, type = 'png') {
        if (!['png', 'svg'].includes(type)) {
            throw new Error('Invalid type. Only png and svg are supported.');
        }
        const RendererInstance = new this();
        RendererInstance.type = type;
        await RendererInstance.init(engineType, disableHeadless);
        return RendererInstance;
    }
    getEngine(engine) {
        switch (engine) {
            case "chromium" /* CarbonCLIEngineFlagEnum.chromium */: {
                return chromium;
            }
            case "firefox" /* CarbonCLIEngineFlagEnum.firefox */: {
                return firefox;
            }
            case "webkit" /* CarbonCLIEngineFlagEnum.webkit */: {
                return webkit;
            }
            default: {
                return chromium;
            }
        }
    }
    async init(engineType, hasHeadlessDisabled) {
        this.browser = await this.getEngine(engineType).launch({
            headless: !hasHeadlessDisabled,
        });
        this.page = await this.browser.newPage(this.pageOptions);
    }
    async navigate(url) {
        await this.page.goto(url);
        await (await this.page.waitForSelector('#export-menu'))?.click();
        await (await this.page.$(`#export-${this.type}`))?.click();
    }
    async setCustomTheme(highlights, theme = CARBON_CUSTOM_THEME) {
        await this.page.addInitScript(({ highlights, theme, CARBON_LOCAL_STORAGE_KEY }) => {
            const themes = [
                {
                    id: theme,
                    name: theme,
                    highlights,
                    custom: true,
                },
            ];
            window.localStorage.setItem(CARBON_LOCAL_STORAGE_KEY, JSON.stringify(themes));
        }, 
        // Passing this in as the 2nd parameter is crucial, see:
        // https://github.com/microsoft/playwright/issues/6258#issuecomment-1030704374
        { highlights, theme, CARBON_LOCAL_STORAGE_KEY });
    }
    async download(url, saveDirectory = process.cwd()) {
        try {
            const queuedDownloadEvent = this.page.waitForEvent('download');
            await this.navigate(url);
            await (await queuedDownloadEvent)?.saveAs(`${saveDirectory}/carbon.${this.type}`);
        }
        catch (e) {
            throw new Error(e.message);
        }
        finally {
            await this.browser.close();
        }
    }
}
